use async_trait::async_trait;
use reqwest::Method;
use serenity::model::interactions::application_command::{
    ApplicationCommandInteraction, ApplicationCommandOptionType,
};

use geoffrey_models::models::locations::Location;
use geoffrey_models::models::parameters::remove_item_params::RemoveItemParameters;

use crate::bot::arg_parse::option_to_string;
use crate::bot::commands::{BotCommand, CommandError};
use crate::bot::formatters::display_loc_full;
use crate::bot::lang::PLAYER_DOES_NOT_HAVE_MATCHING_SHOP;
use geoffrey_models::models::response::api_error::GeoffreyAPIError;
use serenity::builder::CreateApplicationCommand;

pub struct RemoveItemCommand;

#[async_trait]
impl BotCommand for RemoveItemCommand {
    type ApiParams = RemoveItemParameters;
    type ApiResp = Location;

    fn command_name() -> String {
        "remove_item".to_string()
    }

    fn request_type() -> Method {
        Method::POST
    }

    fn custom_err_resp(e: &CommandError) -> Option<String> {
        match e {
            CommandError::GeoffreyApi(GeoffreyAPIError::EntryNotFound) => {
                Some(PLAYER_DOES_NOT_HAVE_MATCHING_SHOP.to_string())
            }
            _ => None,
        }
    }

    fn create_app_command(command: &mut CreateApplicationCommand) -> &mut CreateApplicationCommand {
        command
            .name(Self::command_name())
            .description("Remove an item from a shop")
            .create_option(|option| {
                option
                    .name("shop_name")
                    .description("Shop to list the item at")
                    .kind(ApplicationCommandOptionType::String)
                    .required(true)
            })
            .create_option(|option| {
                option
                    .name("item_name")
                    .description("Name of the item to remove")
                    .kind(ApplicationCommandOptionType::String)
                    .required(true)
            })
    }

    async fn process_arguments(
        command_interaction: ApplicationCommandInteraction,
    ) -> Result<Self::ApiParams, CommandError> {
        let options = command_interaction.data.options;

        Ok(Self::ApiParams::new(
            option_to_string(options.get(0), "shop_name")?,
            option_to_string(options.get(1), "item_name")?,
        ))
    }

    fn build_response(resp: Self::ApiResp) -> String {
        format!(
            "**{}** has been updated:\n{}",
            resp.name,
            display_loc_full(&resp)
        )
    }
}
